@@ -95,11 +95,11 @@ test_that("PF_smooth gives same results", {
     n_fixed_terms_in_state_vec = 0,
     X = t(X_Y$X), fixed_terms = t(X_Y$fixed_terms),
     tstart = X_Y$Y[, 1], tstop = X_Y$Y[, 2], R = diag(1, ncol(Q)), Q_0 = Q_0,
-    fixed_parems = numeric(), Q = Q, a_0 = a_0, Q_tilde = diag(1e-2, n_vars + 1),
+    fixed_params = numeric(), Q = Q, a_0 = a_0, Q_tilde = diag(1e-2, n_vars + 1),
     risk_obj = risk_set, F = diag(1, n_vars + 1), n_max = 10, n_threads = 1,
     N_fw_n_bw = 20, N_smooth = 100, N_first = 100, N_smooth_final = 100,
-    forward_backward_ESS_threshold = NULL, debug = 0,
-    method = "bootstrap_filter",
+    forward_backward_ESS_threshold = NULL, debug = 0, ftol_rel = 1e-8,
+    method = "bootstrap_filter", covar_fac = -1,
     smoother = "Fearnhead_O_N", model = "logit", type = "RW", nu = 0L)
 
   fw_args <- list(
@@ -554,6 +554,28 @@ test_that("'get_cloud_means' and 'get_cloud_quantiles' gives previous results",
   expect_true(!isTRUE(all.equal(q1, q7)))
 })
 
+test_that("'get_ancestors' yields the correct result", {
+  skip_on_cran()
+  skip_if(!dir.exists("previous_results/local_tests"))
+
+  pf_fit <- read_to_test("local_tests/PF_head_neck")
+  class(pf_fit) <- "PF_EM"
+
+  cpp_out <- test_get_ancestors(pf_fit$clouds)
+
+  fw <- pf_fit$clouds$forward_clouds
+  n_ele <- length(fw)
+  correct <- list()
+  correct[[n_ele]] <- seq_len(length(fw[[n_ele]]$weights))
+  for(i in rev(seq_len(n_ele - 1L)))
+    correct[[i]] <- unique(fw[[i + 1L]]$parent_idx[correct[[i + 1L]]])
+
+  for(i in seq_along(correct))
+    correct[[i]] <- correct[[i]] - 1L
+
+  expect_equal(correct, cpp_out)
+})
+
 test_that("´est_params_dens´ gives the same as a R version", {
   skip_on_cran()
   fit <- suppressWarnings(PF_EM(
@@ -595,10 +617,10 @@ test_that("´est_params_dens´ gives the same as a R version", {
 
   qr_obj <- qr(wX)
   qty <- qr.qty(qr_obj, wY)[1:ncol(wX), ]
-  Q <- (crossprod(wY) - crossprod(qty)) / (length(clouds) - 1)
+  Q <- (crossprod(wY) - crossprod(qty)) / length(clouds)
   expect_equal(Q, fit$Q, check.attributes = FALSE)
 
-  Q <- crossprod(wY - wX %*% lm_fit$coefficients) / (length(clouds) - 1)
+  Q <- crossprod(wY - wX %*% lm_fit$coefficients) / length(clouds)
   expect_equal(Q, fit$Q, check.attributes = FALSE)
 })
 
@@ -692,18 +714,6 @@ test_that("A few iterations with `type = \"VAR\"' yields the same as before", {
   # Fmat <- matrix(c(.8, 0, 0, .8), 2)
   # Rmat <- diag(1    , 2)
   # Qmat <- diag(.33^2, 2)
-  # get_Q_0 <- function(Qmat, Fmat){
-  #   # see https://math.stackexchange.com/q/2854333/253239
-  #   eg  <- eigen(Fmat)
-  #   las <- eg$values
-  #   if(any(abs(las) >= 1))
-  #     stop("Divergent series")
-  #   U   <- eg$vectors
-  #   U_t <- t(U)
-  #   T.  <- crossprod(U, Qmat %*% U)
-  #   Z   <- T. / (1 - tcrossprod(las))
-  #   solve(U_t, t(solve(U_t, t(Z))))
-  # }
   # Q_0  <- get_Q_0(Qmat, Fmat)
   # beta <- c(-6.5, -2)
   #
@@ -770,20 +780,6 @@ test_that("PF_EM gives the same with restricted and unrestricted model when we e
   n_obs     <- 200L
   n_periods <- 100L
 
-  # # function to find Q_0
-  # get_Q_0 <- function(Qmat, Fmat){
-  #   # see https://math.stackexchange.com/q/2854333/253239
-  #   eg  <- eigen(Fmat)
-  #   las <- eg$values
-  #   if(any(abs(las) >= 1))
-  #     stop("Divergent series")
-  #   U   <- eg$vectors
-  #   U_t <- t(U)
-  #   T.  <- crossprod(U, Qmat %*% U)
-  #   Z   <- T. / (1 - tcrossprod(las))
-  #   solve(U_t, t(solve(U_t, t(Z))))
-  # }
-  #
   # Fmat <- matrix(c(.9, 0, 0, .9), 2)
   # Rmat <- diag(1    , 2)
   # Qmat <- diag(.33^2, 2)
@@ -849,7 +845,7 @@ test_that("PF_EM gives the same with restricted and unrestricted model when we e
     pf_non_restrict_Fear$Q, pf_restrict_Fear$Q, tolerance = 1e-5)
 })
 
-test_that("type = 'VAR' works with non-zero mean for with a single term and gives previous results", {
+test_that("type = 'VAR' works with non-zero mean with a single term and gives previous results", {
   # had somes issues when with an example like this
   set.seed(30520116)
   pf_Fear <- suppressWarnings(PF_EM(
@@ -983,8 +979,8 @@ test_that("`get_Q_0` returns a real matrix also when `Fmat` has a complex eigend
   expect_true(!is.complex(out))
   expect_equal(
     out,
-    structure(c(0.0621064332808536, -0.0111136683871378, -0.0111136683871378,
-                0.0250726831993073), .Dim = c(2L, 2L)))
+    structure(c(0.065269831500739, 0.00500931948325941, 0.0050093194832594,
+                0.031570480294995), .Dim = c(2L, 2L)))
 })
 
 test_that("'PF_forward_filter' gives the same as 'PF_EM' when it should", {
@@ -1081,3 +1077,566 @@ test_that("'PF_forward_filter' gives the same as 'PF_EM' when it should", {
   filter_res <- PF_forward_filter(pf_em_res, N_fw = 25, N_first = 25)
   expect_equal(filter_res$forward_clouds, pf_em_res$clouds$forward_clouds)
 })
+
+if(dir.exists("pf-internals"))
+  invisible(sapply(list.files("pf-internals", full.names = TRUE), source))
+
+sim_test_pf_internal <- function(n, p, fam, state){
+  X <- matrix(runif(n * p, -1, 1), nrow = p)
+  offset <- runif(n, -1, 1)
+
+  tstart <- numeric(n)
+  tstop <- tstart + 1 + runif(n, max = 3)
+
+  bin_start <- 1
+  bin_stop  <- 2
+
+  eta <- drop(state %*% X) + offset
+  if(fam == "binomial"){
+    y <- 1/(1 + exp(-eta)) > runif(n)
+
+  } else if(fam == "cloglog"){
+    y <- -expm1(-exp(eta)) > runif(n)
+
+  } else if(fam == "poisson"){ # TODO: poisson is missleading
+    y <- rexp(length(eta), exp(eta))
+    dt <- pmin(tstop, bin_stop) - pmax(tstart, bin_start)
+    is_event <- y <= dt
+    tstop[is_event] <- pmax(tstart[is_event], bin_start) + y[is_event]
+    y <- Surv(pmin(y, dt), is_event)
+
+  }
+
+  list(y = y, eta = eta, bin_start = bin_start,
+       bin_stop = bin_stop, tstart = tstart, tstop = tstop,
+       offset = offset, X = X)
+}
+
+test_that("'state_fw' gives correct results", {
+  skip_if(!dir.exists("pf-internals"))
+  skip_if_not_installed("mvtnorm")
+
+  parent <- c(-.25, -.1)
+  parent1 <- c(.3, .18)
+  chi <- c(.2, -.05)
+  chi1 <- c(-.2, -.07)
+  F. <- matrix(c(.9, .4, 0, .8), nrow = 2)
+  Q <- matrix(c(.8, .4, .4, .4), nrow = 2)
+
+  obj <- fw(parent = parent, F. = F., Q = Q)
+
+  cpp_out <- check_state_fw(
+    parent = parent, parent1 = parent1, child = chi, child1 = chi1,
+    F = F., Q = Q)
+
+  require(mvtnorm)
+  expect_equal(dmvnorm(chi, F. %*% parent, Q, TRUE), cpp_out$log_dens_func)
+  expect_true(cpp_out$is_mvn)
+  expect_true(!cpp_out$is_grad_z_hes_const)
+  expect_equal(cpp_out$dim, length(parent))
+
+  expect_equal(obj$f(chi), cpp_out$log_dens)
+  expect_equal(obj$f(chi1), cpp_out$log_dens1)
+
+  expect_equal(obj$deriv(chi), cpp_out$gradient)
+  expect_equal(obj$deriv(chi1), cpp_out$gradient1)
+
+  expect_equal(obj$deriv_z(parent), cpp_out$gradient_zero)
+  expect_equal(obj$deriv_z(parent1), cpp_out$gradient_zero1)
+
+  expect_equal(obj$n_hessian(chi), cpp_out$neg_Hessian)
+  expect_equal(obj$n_hessian(chi1), cpp_out$neg_Hessian1)
+})
+
+test_that("'state_bw' gives correct results", {
+  skip_if(!dir.exists("pf-internals"))
+  skip_if_not_installed("mvtnorm")
+
+  parent <- c(-.25, -.1)
+  parent1 <- c(.3, .18)
+  chi <- c(.2, -.05)
+  chi1 <- c(-.2, -.07)
+  F. <- matrix(c(.9, .4, 0, .8), nrow = 2)
+  Q <- matrix(c(.8, .4, .4, .4), nrow = 2)
+
+  obj <- bw(child = chi, F. = F., Q = Q)
+
+  cpp_out <- check_state_bw(
+    parent = parent, parent1 = parent1, child = chi, child1 = chi1,
+    F = F., Q = Q)
+
+  require(mvtnorm)
+  expect_equal(dmvnorm(chi, F. %*% parent, Q, TRUE), cpp_out$log_dens_func)
+  expect_true(cpp_out$is_mvn)
+  expect_true(!cpp_out$is_grad_z_hes_const)
+  expect_equal(cpp_out$dim, length(chi))
+  expect_equal(obj$f(parent), cpp_out$log_dens)
+  expect_equal(obj$f(parent1), cpp_out$log_dens1)
+
+  expect_equal(obj$deriv(parent), cpp_out$gradient)
+  expect_equal(obj$deriv(parent1), cpp_out$gradient1)
+
+  expect_equal(obj$deriv_z(chi), cpp_out$gradient_zero)
+  expect_equal(obj$deriv_z(chi1), cpp_out$gradient_zero1)
+
+  expect_equal(obj$n_hessian(chi), cpp_out$neg_Hessian)
+  expect_equal(obj$n_hessian(chi1), cpp_out$neg_Hessian1)
+})
+
+test_that("'artificial_prior' gives correct results", {
+  skip_if(!dir.exists("pf-internals"))
+  skip_if_not_installed("mvtnorm")
+
+  state <- c(-.25, -.1)
+  F. <- matrix(c(.9, .4, 0, .8), nrow = 2)
+  Q <- matrix(c(.8, .4, .4, .4), nrow = 2)
+  m_0 <- c(.2, .3)
+
+  prio <- prior(F. = F., Q = Q, Q_0 = Q, mu_0 = m_0)
+
+  t1 <- 1L
+  t2 <- 4L
+  t3 <- 20L
+  ts <- c(t1, t2, t3)
+
+  cpp_out <- check_artificial_prior(
+    state = state, F = F., Q = Q, m_0 = m_0, Q_0 = Q, t1 = t1, t2 = t2,
+    t3 = t3)
+
+  for(i in seq_along(cpp_out)){
+    cpp_o <- cpp_out[[i]]
+    p <- prio(ts[i])
+
+    expect_true(cpp_o$is_mvn)
+    expect_true(cpp_o$is_grad_z_hes_const)
+    expect_equal(cpp_o$dim, length(state))
+    expect_equal(cpp_o$log_dens, p$f(state))
+    expect_equal(cpp_o$gradient, p$deriv(state))
+    expect_equal(cpp_o$gradient_zero, p$deriv_z(state))
+    expect_equal(cpp_o$neg_Hessian, p$n_hessian(state))
+  }
+})
+
+test_that("'observational_cdist' gives correct results", {
+  skip_if(!dir.exists("pf-internals"))
+  skip_if_not_installed("mvtnorm")
+
+  p <- 3
+  state1 <- 1:p - p / 2
+  state2 <- state1 + 1
+  set.seed(1)
+
+  for(l in list(
+    list("binomial", binomial()), list("cloglog", binomial("cloglog")),
+    list("poisson", "exponential"))){
+    fam <- l[[1]]
+    o <- sim_test_pf_internal(n = 50L, p = 3L, fam = fam, state = state1)
+
+    obj <- with(o, odist(y, t(X), offset, l[[2]]))
+
+    y_use <- if(fam == "poisson") o$y[, 2] else o$y
+    cpp_out <- with(o, check_observational_cdist(
+      X = X, is_event = y_use, offsets = offset, tstart = tstart,
+      tstop = tstop, bin_start = bin_start, bin_stop = bin_stop,
+      multithreaded = FALSE, fam = fam, state = state1,
+      state1 = state2))
+
+    cpp_out_mult <- with(o, check_observational_cdist(
+      X = X, is_event = y_use, offsets = offset, tstart = tstart,
+      tstop = tstop, bin_start = bin_start, bin_stop = bin_stop,
+      multithreaded = TRUE, fam = fam, state = state1,
+      state1 = state2))
+    expect_equal(cpp_out, cpp_out_mult)
+
+    expect_true(!cpp_out$is_mvn)
+    expect_equal(cpp_out$dim, length(state1))
+
+    expect_equal(cpp_out$log_dens, obj$f(state1))
+    expect_equal(cpp_out$log_dens1, obj$f(state2))
+
+    expect_equal(drop(cpp_out$gradient), obj$deriv(state1))
+    expect_equal(drop(cpp_out$gradient1), obj$deriv(state2))
+
+    expect_equal(cpp_out$neg_Hessian, obj$n_hessian(state1))
+    expect_equal(cpp_out$neg_Hessian1, obj$n_hessian(state2))
+  }
+})
+
+test_that("combining forward and backwards works", {
+  skip_if(!dir.exists("pf-internals"))
+  skip_if_not_installed("mvtnorm")
+
+  p1 <- c(-.5, 0)
+  p2 <- c(-.2, .2)
+  c1 <- p2
+  c2 <- p1
+  x <- c(-.33, .33)
+  F. <- matrix(c(.9, .4, 0, .8), nrow = 2)
+  Q <- matrix(c(.8, .4, .4, .4), nrow = 2)
+
+  for(nu in c(-1L, 1L, 4L)){
+    cpp_out <- check_fw_bw_comb(
+      F = F., Q = Q, parent = p1, parent1 = p2,
+      grand_child = c1, grand_child1 = c2, x = x, nu = nu)
+
+    #####
+    Sig_org <- Sig <- solve(solve(Q) + crossprod(F., solve(Q, F.)))
+    m1 <- Sig %*% (solve(Q, F. %*% p1) + crossprod(F., solve(Q, c1)))
+
+    if(nu <= 2L){
+      lfunc <- dmvnorm
+
+    } else {
+      Sig <- Sig * (nu - 2L) / nu
+      lfunc <- function(x, m, s, log)
+        dmvt(x = x, delta = m, sigma = s, df = nu, log = TRUE)
+
+    }
+
+    cpp_1 <- cpp_out[[1L]]
+    expect_equal(cpp_1$mean, m1)
+    expect_equal(cpp_1$covar, Sig)
+    require(mvtnorm)
+    expect_equal(cpp_1$log_density, lfunc(x, m1, Sig, log = TRUE))
+
+    #####
+    m2 <- Sig_org %*% (solve(Q, F. %*% p2) + crossprod(F., solve(Q, c2)))
+
+    cpp_2 <- cpp_out[[2L]]
+    expect_equal(cpp_2$mean, m2)
+    expect_equal(cpp_2$covar, Sig)
+    expect_equal(cpp_2$log_density, lfunc(x, m2, Sig, log = TRUE))
+  }
+})
+
+test_that("combining prior and backwards works", {
+  skip_if(!dir.exists("pf-internals"))
+  skip_if_not_installed("mvtnorm")
+
+  p <- c(-.5, 0)
+  c1 <- c(-.4, 1)
+  c2 <- c(.2, .2)
+  m_0 <- c(0, .4)
+  F. <- matrix(c(.9, .4, 0, .8), nrow = 2)
+  Q <- matrix(c(.8, .4, .4, .4), nrow = 2)
+  Q_0 <- Q * 1.1
+  t1 <- 1L
+  t2 <- 10L
+
+  bwo <- bw(child = c1, F. = F., Q = Q)
+  prio <- prior(F. = F., Q = Q, Q_0 = Q_0, mu_0 = m_0)
+
+  cpp_out <- check_prior_bw_comb(
+    F = F., Q = Q, m_0 = m_0, Q_0 = Q_0, child = c1, child1 = c2,
+    parent = p, t1 = t1, t2 = t2)
+
+  for(t. in c(t2, t1)){
+    co <- cpp_out[[as.character(t.)]]
+    app <- approximator(prio(t.), bwo, start = c1)
+
+    o1 <- app(NULL, c1)
+    o2 <- app(NULL, c2)
+    expect_equal(drop(co$mean1), o1$mu)
+    expect_equal(drop(co$mean2), o2$mu)
+
+    expect_equal(co$covar1, o1$Sig)
+    expect_equal(co$covar2, o2$Sig)
+
+    expect_equal(co$log_dens1, o1$p_ldens(p))
+    expect_equal(co$log_dens2, o2$p_ldens(p))
+  }
+})
+
+test_that("mode approximations give expected result", {
+  skip_if(!dir.exists("pf-internals"))
+  skip_if_not_installed("mvtnorm")
+
+  p <- c(-.5, 0)
+  c1 <- c(-.4, 1)
+  c2 <- c(.2, .2)
+  m_0 <- c(0, .4)
+  F. <- matrix(c(.9, .4, 0, .8), nrow = 2)
+  Q <- matrix(c(.8, .4, .4, .4), nrow = 2)
+  Q_0 <- Q * 1.1
+  t1 <- 3L
+
+  bwo <- bw(child = c1, F. = F., Q = Q)
+  prio <- prior(F. = F., Q = Q, Q_0 = Q_0, mu_0 = m_0)(t1)
+
+  for(l in list(
+    list("poisson", "exponential"),
+    list("cloglog", binomial("cloglog")), list("binomial", binomial()))){
+    fam <- l[[1]]
+    set.seed(1)
+    o <- sim_test_pf_internal(n = 50L, p = 2L, fam = fam, state = p)
+
+    y_dist <- with(o, odist(y, t(X), offset, l[[2]]))
+
+    app1 <- approximator(bwo, prio, start = c1)
+    app <- approximator(bwo, prio, y_dist, start = app1(c1, NULL)$mu)
+    o1 <- app(c1, NULL)
+    o2 <- app(c2, NULL)
+    Q_xtra <- Q
+    Q_xtra[] <- 0
+
+    y_use <- if(fam == "poisson") o$y[, 2] else o$y
+    cpp_out <- with(o, check_prior_bw_state_comb(
+      X = X, is_event = y_use, offsets = offset, tstart = tstart,
+      tstop = tstop, bin_start = bin_start, bin_stop = bin_stop, fam = fam,
+      F = F., Q = Q, Q_0 = Q_0, m_0 = m_0, child = c1, child1 = c2, parent = p,
+      t1 = t1, Q_xtra = Q_xtra))
+
+    expect_equal(drop(cpp_out$mean1), o1$mu)
+    expect_equal(drop(cpp_out$mean2), o2$mu)
+
+    expect_equal(cpp_out$covar1, o1$Sig)
+    expect_equal(cpp_out$covar2, o2$Sig)
+
+    expect_equal(cpp_out$log_dens1, o1$p_ldens(p))
+    expect_equal(cpp_out$log_dens2, o2$p_ldens(p))
+  }
+
+  #####
+  # with extra covariance matrix term
+  Q_xtra <- diag(sqrt(.3), 2)
+  set.seed(1)
+  o <- sim_test_pf_internal(n = 50L, p = 2L, fam = fam, state = p)
+
+  y_dist <- with(o, odist(y, t(X), offset, l[[2]]))
+
+  app1 <- approximator(bwo, prio, start = c1)
+  app <- approximator(bwo, prio, y_dist, start = app1(c1, NULL)$mu)
+  o1 <- app(c1, NULL)
+  o2 <- app(c2, NULL)
+
+  y_use <- if(fam == "poisson") o$y[, 2] else o$y
+  cpp_out <- with(o, check_prior_bw_state_comb(
+    X = X, is_event = y_use, offsets = offset, tstart = tstart,
+    tstop = tstop, bin_start = bin_start, bin_stop = bin_stop, fam = fam,
+    F = F., Q = Q, Q_0 = Q_0, m_0 = m_0, child = c1, child1 = c2, parent = p,
+    t1 = t1, Q_xtra = Q_xtra))
+
+  expect_equal(drop(cpp_out$mean1), o1$mu)
+  expect_equal(drop(cpp_out$mean2), o2$mu)
+
+  Q_res <- o1$Sig + Q_xtra
+  expect_equal(cpp_out$covar1, Q_res)
+  expect_equal(cpp_out$covar2, Q_res)
+
+  expect_equal(cpp_out$log_dens1, dmvnorm(p, o1$mu, Q_res, TRUE))
+  expect_equal(cpp_out$log_dens2, dmvnorm(p, o2$mu, Q_res, TRUE))
+
+  #####
+  # with extra covariance matrix term and t-distribution
+  nu <- 4
+  cpp_out <- with(o, check_prior_bw_state_comb(
+    X = X, is_event = y_use, offsets = offset, tstart = tstart,
+    tstop = tstop, bin_start = bin_start, bin_stop = bin_stop, fam = fam,
+    F = F., Q = Q, Q_0 = Q_0, m_0 = m_0, child = c1, child1 = c2, parent = p,
+    t1 = t1, Q_xtra = Q_xtra, nu = nu))
+
+  expect_equal(drop(cpp_out$mean1), o1$mu)
+  expect_equal(drop(cpp_out$mean2), o2$mu)
+
+  Q_res <- (o1$Sig + Q_xtra) * (nu - 2) / nu
+  expect_equal(cpp_out$covar1, Q_res)
+  expect_equal(cpp_out$covar2, Q_res)
+
+  expect_equal(cpp_out$log_dens1, dmvt(p, o1$mu, Q_res, nu, TRUE))
+  expect_equal(cpp_out$log_dens2, dmvt(p, o2$mu, Q_res, nu, TRUE))
+
+  #####
+  # with extra covariance matrix term, scaled and t-distribution
+  nu <- 4
+  covar_fac <- 1.5
+  cpp_out <- with(o, check_prior_bw_state_comb(
+    X = X, is_event = y_use, offsets = offset, tstart = tstart,
+    tstop = tstop, bin_start = bin_start, bin_stop = bin_stop, fam = fam,
+    F = F., Q = Q, Q_0 = Q_0, m_0 = m_0, child = c1, child1 = c2, parent = p,
+    t1 = t1, Q_xtra = Q_xtra, nu = nu, covar_fac = covar_fac))
+
+  expect_equal(drop(cpp_out$mean1), o1$mu)
+  expect_equal(drop(cpp_out$mean2), o2$mu)
+
+  Q_res <- (o1$Sig + Q_xtra) * (nu - 2) / nu * covar_fac
+  expect_equal(cpp_out$covar1, Q_res)
+  expect_equal(cpp_out$covar2, Q_res)
+
+  expect_equal(cpp_out$log_dens1, dmvt(p, o1$mu, Q_res, nu, TRUE))
+  expect_equal(cpp_out$log_dens2, dmvt(p, o2$mu, Q_res, nu, TRUE))
+
+  #####
+  # lower threshold in mode approximation
+  ftol_rel <- 1e-3
+  app <- approximator(bwo, prio, y_dist, start = app1(c1, NULL)$mu,
+                      ftol_rel = ftol_rel)
+  o1 <- app(c1, NULL)
+  o2 <- app(c2, NULL)
+  cpp_out <- with(o, check_prior_bw_state_comb(
+    X = X, is_event = y_use, offsets = offset, tstart = tstart,
+    tstop = tstop, bin_start = bin_start, bin_stop = bin_stop, fam = fam,
+    F = F., Q = Q, Q_0 = Q_0, m_0 = m_0, child = c1, child1 = c2, parent = p,
+    t1 = t1, Q_xtra = Q_xtra, ftol_rel = ftol_rel))
+
+  expect_equal(drop(cpp_out$mean1), o1$mu)
+  expect_equal(drop(cpp_out$mean2), o2$mu)
+
+  Q_res <- o1$Sig + Q_xtra
+  expect_equal(cpp_out$covar1, Q_res)
+  expect_equal(cpp_out$covar2, Q_res)
+
+  expect_equal(cpp_out$log_dens1, dmvnorm(p, o1$mu, Q_res, TRUE))
+  expect_equal(cpp_out$log_dens2, dmvnorm(p, o2$mu, Q_res, TRUE))
+})
+
+test_that("'PF_get_score_n_hess' gives the same as an R implementation", {
+  skip_on_cran()
+
+  .lung <- lung[!is.na(lung$ph.ecog), ]
+  # standardize
+  .lung$age <- scale(.lung$age)
+
+  # fit
+  set.seed(43588155)
+  pf_fit <- suppressWarnings(PF_EM(
+    Surv(time, status == 2) ~ ddFixed(ph.ecog) + ddFixed(age) + age,
+    data = .lung, by = 50, id = 1:nrow(.lung), Q_0 = diag(1, 2),
+    Fmat = structure(c(0.96, -0.00045, 0.066, 0.37), .Dim = c(2L, 2L)),
+    Q = structure(c(0.071, -0.055, -0.055, 0.058), .Dim = c(2L, 2L)),
+    max_T = 800, fixed_effects = c(0.36, 0.1), type = "VAR",
+    control = PF_control(
+      N_fw_n_bw = 500, N_first = 2500, N_smooth = 100,
+      n_max = 1, eps = .001, Q_tilde = diag(.2^2, 2), est_a_0 = FALSE,
+      n_threads = max(parallel::detectCores(logical = FALSE), 1))))
+
+  test_logit_func <- function(fit){
+    org_cl <- fit$call
+    ma <- match(
+      c("formula", "data", "by", "max_T", "id", "trace", "model", "order",
+        "fixed", "fixed", "random"), names(org_cl), nomatch = 0L)
+    sta_arg_call <- org_cl[c(1L, ma)]
+
+    # add defaults where needed
+    def_args <- c("trace", "model", "fixed", "random", "order")
+    if(any(is_missing <- !def_args %in% names(sta_arg_call)))
+      sta_arg_call[def_args[is_missing]] <- formals(PF_EM)[def_args[is_missing]]
+    sta_arg_call[[1L]] <- quote(dynamichazard:::.get_PF_static_args)
+
+    static_args <- eval(sta_arg_call)
+    fw <- fit$clouds$forward_clouds
+
+    a_sta_old <- B_sta_old <- a_obs_old <- B_obs_old <- NULL
+    k <- length(fit$F)
+    K <- solve(fit$Q)
+    K1_2 <- K / 2
+    for(i in seq_len(length(fw) - 1L) + 1L){
+      fw_i <- fw[[i]]
+      ps <- fw_i$states
+      par_idx <- fw_i$parent_idx
+      parents <- fw[[i - 1L]]$states[, par_idx]
+
+      R_i <- static_args$risk_obj$risk_sets[[i - 1L]]
+      X_i <- static_args$X[, R_i, drop = FALSE]
+      Z_i <- static_args$fixed_terms[, R_i, drop = FALSE]
+      y_i <- static_args$risk_obj$is_event_in[R_i] == (i - 2L)
+
+      eta <- drop(crossprod(Z_i, fit$fixed_effects))
+
+      #####
+      # state equation
+      a_sta <- matrix(0., ncol(ps), k * 2L)
+      B_sta <- array(0, c(k * 2L, k * 2L, ncol(ps)))
+      r0 <- ps - fit$F %*% parents
+      r1 <- solve(fit$Q, r0)
+      for(j in 1:ncol(ps)){
+        a_sta[j, 1:k] <- tcrossprod(parents[, j], r1[, j])
+        a_sta[j, 1:k + k] <-
+          solve(fit$Q, tcrossprod(r0[, j] * .5, r1[, j]) - diag(.5, k / 2L))
+
+        B_sta[1:k, 1:k, j] <- - kronecker(K, tcrossprod(parents[, j]))
+        off_block <- - kronecker(K, tcrossprod(r1[, j], parents[, j]))
+        B_sta[1:k + k, 1:k, j    ] <- off_block
+        B_sta[1:k    , 1:k + k, j] <- t(off_block)
+        B_sta[1:k + k, 1:k + k, j] <-
+          - kronecker(K, tcrossprod(r1[, j]) - K1_2)
+      }
+
+      if(!is.null(a_sta_old))
+        a_sta <- a_sta + a_sta_old[par_idx, ]
+      if(!is.null(B_sta_old))
+        B_sta <- B_sta + B_sta_old[, , par_idx]
+
+      a_sta_old <- a_sta
+      B_sta_old <- B_sta
+
+      #####
+      # observational equation
+      a_obs <- apply(ps, 2, function(x){
+        eta <- eta + drop(crossprod(X_i, x))
+        exp_eta <- exp(eta)
+        drop(crossprod((exp_eta * (y_i - 1) + y_i) / (1 + exp_eta), t(Z_i)))
+      })
+      if(!is.matrix(a_obs))
+        a_obs <- as.matrix(a_obs) else a_obs <- t(a_obs)
+      B_obs <- apply(ps, 2, function(x){
+        eta <- eta + drop(crossprod(X_i, x))
+        exp_eta <- exp(eta)
+        crossprod(-t(Z_i) * exp(eta) / (1 + exp(eta))^2, t(Z_i))
+      })
+      B_obs <- array(B_obs, dim = c(NCOL(a_obs), NCOL(a_obs), NROW(a_obs)))
+
+      if(!is.null(a_obs_old))
+        a_obs <- a_obs + a_obs_old[par_idx, ]
+      if(!is.null(B_obs_old))
+        B_obs <- B_obs + B_obs_old[, , par_idx]
+
+      a_obs_old <- a_obs
+      B_obs_old <- B_obs
+    }
+
+    #####
+    # state equation
+    ws <- drop(tail(fw, 1)[[1L]]$weights)
+    S_state <- colSums(a_sta * ws)
+
+    info_obj_sta <- matrix(0., NCOL(a_sta), NCOL(a_sta))
+    for(i in seq_along(ws))
+      info_obj_sta <-
+      info_obj_sta + ws[i] * (tcrossprod(a_sta[i, ]) + B_sta[, , i])
+
+    d <- ncol(fit$Q)
+    K <- matrix(0., 2L * d * d, 2L * d * d)
+    K[1:(d * d), 1:(d * d)] <- dynamichazard:::.get_cum_mat(d, d)
+    diag(K[-(1:(d * d)), -(1:(d * d))]) <- 1
+
+    S_state <- drop(K %*% S_state)
+    info_obj_sta <- tcrossprod(K %*% info_obj_sta, K)
+
+    #####
+    # observation equation
+    S_obj <- colSums(a_obs * ws)
+    info_obj_obj <- matrix(0., NCOL(a_obs), NCOL(a_obs))
+    for(i in seq_along(ws))
+      info_obj_obj <-
+      info_obj_obj + ws[i] * (tcrossprod(a_obs[i, ]) + B_obs[, , i])
+
+    list(
+      state = list(score =
+                     S_state, neg_obs_info = tcrossprod(S_state) - info_obj_sta),
+      observation = list(
+        score = S_obj, neg_obs_info = tcrossprod(S_obj) - info_obj_obj))
+  }
+
+  test_out <- test_logit_func(pf_fit)
+  expect_output(func_out <- PF_get_score_n_hess(pf_fit),
+                regexp = "Using '.lung' as the 'data' argument", fixed = TRUE)
+
+  expect_equal(test_out, func_out$get_get_score_n_hess(only_score = FALSE),
+               check.attributes = FALSE)
+
+  # only score
+  test_out$state$neg_obs_info[] <- NA_real_
+  test_out$observation$neg_obs_info[] <- NA_real_
+  expect_equal(test_out, func_out$get_get_score_n_hess(only_score = TRUE),
+               check.attributes = FALSE)
+})
